# AUTOGENERATED! DO NOT EDIT! File to edit: 02_MNL_swissmetro.ipynb (unless otherwise specified).

__all__ = []

# Cell
import pickle
import numpy as np
import pandas as pd
import biogeme.database as db
import biogeme.biogeme as bio
import biogeme.models as models
from biogeme.expressions import Beta

# Cell
#load package...