# AUTOGENERATED! DO NOT EDIT! File to edit: 01_statsmodels_MNL.ipynb (unless otherwise specified).

__all__ = []

# Cell
import numpy as np
import statsmodels.api as sm
import pandas as pd
import patsy